import { createHash, Hash } from 'crypto';
import { createReadStream } from 'fs';
import { unlink } from 'fs/promises';
import { exec } from 'child_process';
import { tmp } from '../tmp';

export class Hashes {
	public static async file (path : string) : Promise<string> {
		return new Promise((resolve : Function, reject : Function) => {
			const hashSum : Hash = createHash('sha256');
			const stream : any = createReadStream(path);
			stream.on('error', (err : Error) => reject(err));
	    	stream.on('data', (chunk : Buffer) => hashSum.update(chunk));
	    	stream.on('end', () => resolve(hashSum.digest('hex')));
		});
	}

	public static string (str : string) : string {
		const sha : Hash = createHash('sha256').update(str);
		return sha.digest('hex');
	}

	public static async  gcode (gcodePath : string) : Promise<string> {
		const dest : string = tmp('.gcode', '3d-gcode-hash-');
		const cmd : string = `cat "${gcodePath}" | grep -v '; generated by PrusaSlicer' > "${dest}"`
		let hash : string;

		try {
			await Hashes.exec(cmd);
		} catch (err) {
			throw err;
		}

		try {
			hash = await Hashes.file(dest);
		} catch (err) {
			throw err;
		}

		try {
			await unlink(dest); 
		} catch (err) {
			throw err;
		}

		return hash;
	}

	private static async exec (cmd : string) {
		return new Promise((resolve : Function, reject : Function) => {
			return exec(cmd, (err : Error, stdout : string | Buffer, stderr : string | Buffer) => {
				if (err) return reject(err);
				return resolve(true);
			});
		});
	}
}

module.exports = { Hashes };